package goamz_test

import (
    .   "gocheck"
    "testing"
    "goamz"
    "time"
    "http"
    "flag"
    "fmt"
    "os"
)

func Test(t *testing.T) {
    TestingT(t)
}

var integration = flag.Bool("int", false, "Enable integration tests")

type IntSuite struct {
    auth goamz.Auth
}

func (s *IntSuite) SetUpSuite(c *C) {
    if !*integration {
        c.Skip("Integration tests not enabled (-int flag)")
    }
    auth, err := goamz.EnvAuth()
    if err != nil {
        c.Fatal(err.String())
    }
    s.auth = auth
}

type HTTPSuite struct {
    request  chan *http.Request
    response chan *FakeResponse
    pending  chan bool
}

const timeout = 5e9
const fakeEndpoint = "http://localhost:4444"

type FakeResponse struct {
    Status  int
    Headers map[string]string
    Body    string
}

func (s *HTTPSuite) SetUpSuite(c *C) {
    s.request = make(chan *http.Request, 64)
    s.response = make(chan *FakeResponse, 64)
    s.pending = make(chan bool, 64)

    url, _ := http.ParseURL(fakeEndpoint)
    go http.ListenAndServe(url.Host, s)

    s.FakeResponse(123, nil, "")
    for {
        // Wait for it to be up.
        resp, _, err := http.Get(fakeEndpoint)
        if err == nil && resp.StatusCode == 123 {
            break
        }
        fmt.Fprintf(os.Stderr, "\nWaiting for fake server to be up... ")
        time.Sleep(1e8)
    }
    fmt.Fprintf(os.Stderr, "done\n\n")
    s.WaitRequest() // Consume dummy request.
}

func (s *HTTPSuite) TearDownTest(c *C) {
    // Flush requests not consumed by the test itself.
    for {
        _, ok := <-s.request
        if !ok {
            break
        }
    }
}

func (s *HTTPSuite) ServeHTTP(w http.ResponseWriter, req *http.Request) {
    s.request <- req
    var resp *FakeResponse
    select {
    case resp = <-s.response:
    case <-time.After(timeout):
        fmt.Fprintf(os.Stderr, "ERROR: Timeout waiting for test to provide response\n")
        resp = &FakeResponse{500, nil, ""}
    }
    if resp.Headers != nil {
        for k, v := range resp.Headers {
            w.SetHeader(k, v)
        }
    }
    if resp.Status != 0 {
        w.WriteHeader(resp.Status)
    }
    w.Write([]byte(resp.Body))
}

func (s *HTTPSuite) WaitRequest() *http.Request {
    select {
    case req := <-s.request:
        req.ParseForm()
        return req
    case <-time.After(timeout):
        panic("Timeout waiting for goamz request")
    }
    panic("unreached")
}

func (s *HTTPSuite) FakeResponse(status int, headers map[string]string, body string) {
    s.response <- &FakeResponse{status, headers, body}
}
